@@ -5,12 +5,14 @@
 //              with the application.
 // ---------------------------------------------------------------------------
 // $Source: src/prelude.rs $
-// $Revision: 2022-01-06T15:46:31+01:00 $
+// $Revision: 2022-02-10T18:40:51+01:00 $
 
 //! This module reexports all macros and types needed to define a quantity.
 
-pub use crate::{Amnt, AmountT, Qty, Quantity, SIPrefix, Unit};
+pub use crate::{Amnt, AmountT, HasRefUnit, Quantity, SIPrefix, Unit, ONE};
 #[cfg(feature = "fpdec")]
 pub use crate::{Dec, Decimal};
-pub use core::ops::{Div, Mul};
+pub use core::cmp::Ordering;
+pub use core::fmt;
+pub use core::ops::{Add, Div, Mul, Sub};
 pub use qty_macros::quantity;
