@@ -5,7 +5,7 @@
 //              with the application.
 // ---------------------------------------------------------------------------
 // $Source: src/si_prefixes.rs $
-// $Revision: 2022-01-04T17:22:16+01:00 $
+// $Revision: 2022-01-26T15:57:39+01:00 $
 
 use ::qty_macros::EnumIter;
 
@@ -151,7 +151,7 @@ impl SIPrefix {
 
     /// Returns the SI prefix with the exponent `exp`, or `None` if there is no
     /// such SI prefix.
-    pub fn from_exp(exp: i8) -> Option<Self> {
+    pub const fn from_exp(exp: i8) -> Option<Self> {
         match exp {
             -24 => Some(SIPrefix::YOCTO),
             -21 => Some(SIPrefix::ZEPTO),
