@@ -5,18 +5,17 @@
 //              with the application.
 // ---------------------------------------------------------------------------
 // $Source: tests/test_qty.rs $
-// $Revision: 2022-01-06T17:00:38+01:00 $
+// $Revision: 2022-02-17T16:39:59+01:00 $
 
 #[cfg(test)]
 mod quantity_with_ref_unit_tests {
     use quantities::prelude::*;
-    use quantities::ONE;
 
     /// Foo, a completely useless quantity
     #[quantity]
     #[ref_unit(A, "a", MEGA)]
     #[unit(B, "b", 0.4)]
-    #[unit(C, "c", 0.01)]
+    #[unit(C, "c", CENTI, 0.01)]
     struct Foo {}
 
     #[test]
@@ -44,6 +43,22 @@ mod quantity_with_ref_unit_tests {
         assert_eq!(iter_units.next(), None);
     }
 
+    #[test]
+    fn test_unit_from_symbol() {
+        assert_eq!(FooUnit::from_symbol("a"), Some(A));
+        assert_eq!(FooUnit::from_symbol("b"), Some(B));
+        assert_eq!(FooUnit::from_symbol("c"), Some(C));
+        assert_eq!(FooUnit::from_symbol("x"), None);
+    }
+
+    #[test]
+    fn test_unit_from_scale() {
+        assert_eq!(FooUnit::from_scale(Amnt!(1)), Some(A));
+        assert_eq!(FooUnit::from_scale(Amnt!(0.4)), Some(B));
+        assert_eq!(FooUnit::from_scale(Amnt!(0.01)), Some(C));
+        assert_eq!(FooUnit::from_scale(Amnt!(10)), None);
+    }
+
     #[test]
     fn test_qty_iter_units() {
         let mut iter_units = Foo::iter_units();
@@ -53,6 +68,42 @@ mod quantity_with_ref_unit_tests {
         assert_eq!(iter_units.next(), None);
     }
 
+    #[test]
+    fn test_fit() {
+        let amnt = Amnt!(0.007);
+        let foo = Foo::_fit(amnt);
+        assert_eq!(foo.amount, amnt / Amnt!(0.01));
+        assert_eq!(foo.unit, C);
+        let amnt = Amnt!(0.07);
+        let foo = Foo::_fit(amnt);
+        assert_eq!(foo.amount, amnt / Amnt!(0.01));
+        assert_eq!(foo.unit, C);
+        let amnt = Amnt!(0.7);
+        let foo = Foo::_fit(amnt);
+        assert_eq!(foo.amount, amnt / Amnt!(0.01));
+        assert_eq!(foo.unit, C);
+        let amnt = Amnt!(7.);
+        let foo = Foo::_fit(amnt);
+        assert_eq!(foo.amount, amnt);
+        assert_eq!(foo.unit, A);
+    }
+
+    #[test]
+    fn test_qty_unit_from_symbol() {
+        assert_eq!(Foo::unit_from_symbol("a"), Some(A));
+        assert_eq!(Foo::unit_from_symbol("b"), Some(B));
+        assert_eq!(Foo::unit_from_symbol("c"), Some(C));
+        assert_eq!(Foo::unit_from_symbol("x"), None);
+    }
+
+    #[test]
+    fn test_qty_unit_from_scale() {
+        assert_eq!(Foo::unit_from_scale(Amnt!(1)), Some(A));
+        assert_eq!(Foo::unit_from_scale(Amnt!(0.4)), Some(B));
+        assert_eq!(Foo::unit_from_scale(Amnt!(0.01)), Some(C));
+        assert_eq!(Foo::unit_from_scale(Amnt!(10)), None);
+    }
+
     #[test]
     fn test_qty() {
         let amnt = Amnt!(17.4);
@@ -228,7 +279,6 @@ mod quantity_with_ref_unit_tests {
 #[cfg(test)]
 mod quantity_without_ref_unit_tests {
     use quantities::prelude::*;
-    use quantities::ONE;
 
     /// Foo, a completely useless quantity
     #[quantity]
@@ -407,3 +457,125 @@ mod quantity_single_unit_tests {
         assert_eq!(qty.unit(), POP);
     }
 }
+
+#[cfg(test)]
+mod derived_quantity_tests {
+    use quantities::prelude::*;
+
+    #[cfg(feature = "fpdec")]
+    macro_rules! amnt_eq {
+        ($x:expr, $y:expr) => {
+            $x == $y
+        };
+    }
+    #[cfg(not(feature = "fpdec"))]
+    macro_rules! amnt_eq {
+        ($x:expr, $y:expr) => {
+            ($x - $y).abs() < 0.000000001
+        };
+    }
+
+    #[quantity]
+    #[ref_unit(Flop, "f")]
+    #[unit(Kiloflop, "kf", 1000., "1000·f")]
+    #[unit(Centiflop, "cf", 0.01, "0.01·f")]
+    struct Foo {}
+
+    #[quantity]
+    #[ref_unit(Emil, "e")]
+    #[unit(Milliemil, "me", 0.001, "0.001·e")]
+    #[unit(Microemil, "µe", 0.000001, "0.000001·e")]
+    #[unit(Kiloemil, "ke", 1000., "1000·e")]
+    struct Bar {}
+
+    #[quantity(Foo * Bar)]
+    #[ref_unit(Bazoo, "b", "1·f·e")]
+    #[unit(Millibazoo, "mb", 0.001, "0.001·b")]
+    #[unit(Microbazoo, "µb", 0.000001, "0.000001·b")]
+    #[unit(Kilobazoo, "kb", 1000., "1000·b")]
+    struct Baz {}
+
+    #[quantity(Foo / Bar)]
+    #[ref_unit(Qoox, "Q", "1·f/e")]
+    #[unit(Flop_per_Emil, "f/e", 1., "1·f/e")]
+    #[unit(Milliqoox, "mQ", 0.001, "0.001·Q")]
+    #[unit(Microqoox, "µQ", 0.000001, "0.000001·Q")]
+    #[unit(Kiloqoox, "kQ", 1000., "1000·Q")]
+    struct Qoo {}
+
+    #[test]
+    fn test_qty() {
+        let amnt = Amnt!(17.4);
+        let unit = BazUnit::Microbazoo;
+        let qty = Baz::new(amnt, unit);
+        assert_eq!(qty.amount(), amnt);
+        assert_eq!(qty.unit(), unit);
+        let qty = amnt * unit;
+        assert_eq!(qty.amount(), amnt);
+        assert_eq!(qty.unit(), unit);
+    }
+
+    fn check_qty_mul_qty(x: Foo, y: Bar, r: Baz) {
+        let z = x * y;
+        assert!(amnt_eq!(z.amount(), r.amount()));
+        assert_eq!(z.unit(), r.unit());
+        let z = y * x;
+        assert!(amnt_eq!(z.amount(), r.amount()));
+        assert_eq!(z.unit(), r.unit());
+        // reverse divs
+        let z = (r / x).convert(y.unit()).unwrap();
+        assert!(amnt_eq!(z.amount(), y.amount()));
+        assert_eq!(z.unit(), y.unit());
+        let z = (r / y).convert(x.unit()).unwrap();
+        assert!(amnt_eq!(z.amount(), x.amount()));
+        assert_eq!(z.unit(), x.unit());
+    }
+
+    #[test]
+    fn test_qty_mul_qty() {
+        check_qty_mul_qty(
+            Amnt!(17.4) * FLOP,
+            Amnt!(3.) * EMIL,
+            Amnt!(17.4) * Amnt!(3.) * BAZOO,
+        );
+        check_qty_mul_qty(
+            Amnt!(14.52) * KILOFLOP,
+            Amnt!(0.47) * MICROEMIL,
+            Amnt!(14.52) * Amnt!(0.47) * MILLIBAZOO,
+        );
+        check_qty_mul_qty(
+            Amnt!(14.52) * CENTIFLOP,
+            Amnt!(0.47) * MICROEMIL,
+            Amnt!(14.52) * Amnt!(0.47) * Amnt!(0.01) * MICROBAZOO,
+        );
+    }
+
+    fn check_qty_div_qty(x: Foo, y: Bar, r: Qoo) {
+        let z = x / y;
+        assert!(amnt_eq!(z.amount(), r.amount()));
+        assert_eq!(z.unit(), r.unit());
+        // reverse mul
+        let z = (r * y).convert(x.unit()).unwrap();
+        assert!(amnt_eq!(z.amount(), x.amount()));
+        assert_eq!(z.unit(), x.unit());
+    }
+
+    #[test]
+    fn test_qty_div_qty() {
+        check_qty_div_qty(
+            Amnt!(17.4) * FLOP,
+            Amnt!(3.) * EMIL,
+            Amnt!(17.4) / Amnt!(3.) * QOOX,
+        );
+        check_qty_div_qty(
+            Amnt!(14.52) * KILOFLOP,
+            Amnt!(3.3) * MILLIEMIL,
+            (Amnt!(14.52) / Amnt!(3.3)) * Amnt!(1000.) * KILOQOOX,
+        );
+        check_qty_div_qty(
+            Amnt!(14.52) * CENTIFLOP,
+            Amnt!(3.3) * KILOEMIL,
+            Amnt!(14.52) / Amnt!(3.3) * Amnt!(10.) * MICROQOOX,
+        );
+    }
+}
